require "jekyll"
require "nokogiri"

class JekyllComponent < Liquid::Block
	def initialize(tag_name, input, parse_context)
		super
		@variables	= {}
	end

	def defaults
		{}
	end

	def handle_defaults
		@variables.merge!(defaults)
	end

	# Options are passed in via the `<options>` tag and should be in YAML
	# format. Towards the bottom of the block we remove the options node from
	# the nokogiri object so that it doesn't get counted as one of the children.
	def handle_options
		options = @doc.at_css('options')
		if options
			stripped = options.text.lines.map { | line | line.gsub(/\G[\t ]/, "  ") }.join("\n")
			# If an empty set of options tags are used then YAML.load with
			# return false which breaks the merge method, so we wrap it in an if
			# statement.
			if yaml = YAML.load(stripped)
				@variables.merge!(yaml)
			end
			options.remove
		end
	end

	# Children are all the elements within the liquid block tags except for the
	# options. Note that each child is an HTML string rather than a nokogiri
	# object so they're best suited for simply rendering out.
	def handle_children
		@variables['children'] = []
		@doc.element_children.each do | child |
			@variables['children'] << child.to_html
		end
	end

	def validate_path
		unless @path
			message = "You must set a @path for your component."
			raise(message)
		end
	end

	# Loads whatever liquid template file you specify in the component's
	# options. If you leave the 'template' attribute blank then we will use
	# '1.liquid' by default. Remember to set the path as an instance variable
	# inside your component eg: `@path = "/components/my-component"`
	def handle_template
		path = File.expand_path('../', __dir__)
		path += "/"
		path += @path
		path += '/templates/'
		path += "#{@variables.dig('template') || 1}"
		path += '.liquid'
		@liquid = Liquid::Template.parse(File.read(path))
	end

	# If you have any component specific behaviour then override this method and
	# it'll be run during the setup method after getting the defaults and
	# options, but before getting the children.
	# A common use case is to grab a particular nokogiri node from the @doc,
	# make it available in the @variables, then remove it from the @doc similar
	# to how the options method works.
	def custom_setup
		return false
	end

	def setup
		validate_path
		handle_defaults
		handle_options
		custom_setup
		handle_children
		handle_template
	end

	# For basic components which require only the standard options, children,
	# defaults etc just 'super' this method in your component.
	def render(context)
		@text		= super
		@doc		= Nokogiri::HTML::DocumentFragment.parse(@text)

		setup

		return @liquid.render(@variables)
	end
end

